from apiclient import errors


def ModifyMessage(service, msg_id, msg_labels):
    """Modify the Labels on the given Message.

    Args:
    service: Authorized Gmail API service instance.
    msg_id: The id of the message required.
    msg_labels: The change in labels.

    Returns:
    Modified message, containing updated labelIds, id and threadId.
    """
    try:
        message = service.users().messages().modify(userId="me", id=msg_id,
                                                    body=msg_labels).execute()
        return message
    except errors.HttpError as error:
        print('An error occurred: %s' % error)


import base64
import email
from apiclient import errors


def GetMimeMessage(service, msg_id):
    """Get a Message and use it to create a MIME Message.

    Args:
        service: Authorized Gmail API service instance.
        msg_id: The ID of the Message required.

    Returns:
        A MIME Message, consisting of data from Message.
    """
    try:
        message = service.users().messages().get(userId="me", id=msg_id,
                                                format='raw').execute()

        # Decode raw message to bytes
        msg_bytes = base64.urlsafe_b64decode(message['raw'])

        # Transform raw bytes into MIME Message
        msg_mime = email.message_from_bytes(msg_bytes)

        return msg_mime
    except errors.HttpError as error:
        print('An error occurred: %s' % error)


def ListMessagesMatchingQuery(service, query=''):
    """List all Messages of the user's mailbox matching the query.

    Args:
        service: Authorized Gmail API service instance.
        query: String used to filter messages returned.
        Eg.- 'from:user@some_domain.com' for Messages from a particular sender.

    Returns:
        List of Messages that match the criteria of the query. Note that the
        returned list contains Message IDs, you must use get with the
        appropriate ID to get the details of a Message.
    """
    try:
        response = service.users().messages().list(userId="me",
                                               q=query).execute()
        messages = []
        if 'messages' in response:
            messages.extend(response['messages'])

        while 'nextPageToken' in response:
            page_token = response['nextPageToken']
            response = service.users().messages().list(userId="me", q=query,
                                                pageToken=page_token).execute()
            messages.extend(response['messages'])

        return messages
    except errors.HttpError as error:
        print('An error occurred: %s' % error)


def ListMessagesWithLabels(service, label_ids=[]):
    """List all Messages of the user's mailbox with label_ids applied.

    Args:
        service: Authorized Gmail API service instance.
        label_ids: Only return Messages with these labelIds applied.

    Returns:
        List of Messages that have all required Labels applied. Note that the
        returned list contains Message IDs, you must use get with the
        appropriate id to get the details of a Message.
    """
    try:
        response = service.users().messages().list(userId="me",
                                                labelIds=label_ids).execute()
        messages = []
        if 'messages' in response:
            messages.extend(response['messages'])

        while 'nextPageToken' in response:
            page_token = response['nextPageToken']
            response = service.users().messages().list(userId="me",
                                                        labelIds=label_ids,
                                                        pageToken=page_token).execute()
            messages.extend(response['messages'])

        return messages
    except errors.HttpError as error:
        print('An error occurred: %s' % error)



import re
import email.message as email_msg

import snoozingmail.gmail.creds as creds
import snoozingmail.gmail.read as read_email
import snoozingmail.gmail.send as send_email
import snoozingmail.gmail.modify as modify_email

class Snoozin:
    """The interface into gmail api wrapper

    Has methods to read, send, and label messages
    in the connected gmail account.
    """
    def __init__(self, credentials):
        """Initialize the Snoozin object by creating the gmail
        service using the provided credentials file.

        Args:
            credentials: Local path to client secret json file.
        """
        self.service = creds.get_gmail_service(credentials)

    def get_matching_msgs(self, query):
        """Get message ids for messages that match the given query
        
        Args:
            query: String used to filter messages returned. (ex:
                   'from:user@some_domain.com' for Messages from 
                   a particular sender.)

        Returns:
            List of message ids
        """
        msg_matches = read_email.ListMessagesMatchingQuery(self.service, query)
        return [msg_match["id"] for msg_match in msg_matches]  

    def get_labeled_msgs(self, label_ids):
        """Get message ids for messages that have all the given labels
        
        Args:
            label_ids: list of label ids that must be included (ex: 
                       '['STARRED', 'UNREAD']' for starred unread emails)

        Returns:
            List of message ids
        """
        labeled_msgs = read_email.ListMessagesWithLabels(self.service, label_ids)
        return [labeled_msg["id"] for labeled_msg in labeled_msgs]

    def get_sender(self, msg_id):
        """Get the sender of the given message.

        Args:
            msg_id: The id of the message, which can be used to get details
                    of the message.

        Returns:
            The sender's email address with name ommitted.
        """
        # Get message
        message = read_email.GetMimeMessage(self.service, msg_id)

        # Get email address of sender
        sender_full = message["From"]
        sender_address = re.search('<(.*)>', sender_full).group(1)
        return sender_address

    def mark_msg_read(self, msg_id):
        """ Mark the given message as read (remove UNREAD label).

        Args:
            msg_id: The id of the message, which can be used to get details
                    of the message.
        """
        msg_labels = { 'removeLabelIds': ['UNREAD'] }
        modify_email.ModifyMessage(self.service, msg_id, msg_labels)

    def get_msg_body(self, msg_id):
        """Get the plain text portion of the given message's body.

        Args:
            msg_id: The id of the message, which can be used to get details
                    of the message.

        Returns:
            The plaintext message body if it exists. Otherwise, None.
        """
        # Get message
        message = read_email.GetMimeMessage(self.service, msg_id)

        # Find the plain text part of the message body
        plain_txt_msg = email_msg.Message()
        if not message.is_multipart() and (message.get_content_type() == 'text/plain'):
           plain_txt_msg = message
        elif message.is_multipart():
            # Go through message and capture plain text of message body
            for part in message.walk():
                if part.get_content_type() == 'text/plain':
                    plain_txt_msg = part

        # If we found plain text, decode it and return.
        if plain_txt_msg:
            msg_bytes = plain_txt_msg.get_payload(decode=True)
            return msg_bytes.decode(plain_txt_msg.get_content_charset())
        else:
            return None

    def send(self, to, subject, message_text, file_path=''):
        """Send a message.

        Args:
            to: Email address of the receiver.
            subject: The subject of the email message.
            message_text: The text of the email message.
            file_path: (optional) The path to the file to be attached.

        Returns:
            The message that was sent
        """

        if file_path:
            message = send_email.CreateMessageWithAttachment(to, subject, message_text, file_path)
        else:
            message = send_email.CreateMessage(to, subject, message_text)
        return send_email.SendMessage(self.service, message)



import pickle
import os.path
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request


############# NOTE (this may be outdated...): ###############
# If originally created gmail api access on another computer,
# need to add the argument --noauth_local_webserver to the python file
# after setting credentials.invalid equal


# If modifying these scopes, delete the file token.pickle.
SCOPES = ['https://www.googleapis.com/auth/gmail.modify']


def get_gmail_service(clien_secret_json):
    """Get gmail service using credentials"""
    creds = None
    # The file token.pickle stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('token.pickle'):
        with open('token.pickle', 'rb') as token:
            creds = pickle.load(token)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                clien_secret_json, SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('token.pickle', 'wb') as token:
            pickle.dump(creds, token)

    service = build('gmail', 'v1', credentials=creds)

    return service



import base64
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import mimetypes
import os

from apiclient import errors


def SendMessage(service, message):
    """Send an email message.

    Args:
        service: Authorized Gmail API service instance.
        message: Message to be sent.

    Returns:
        Sent Message.
    """
    try:
        message = (service.users().messages().send(userId="me", body=message)
                .execute())
        return message
    except errors.HttpError as error:
        print('An error occurred: %s' % error)


def CreateMessage(to, subject, message_text):
    """Create a message for an email.

    Args:
        sender: Email address of the sender.
        to: Email address of the receiver.
        subject: The subject of the email message.
        message_text: The text of the email message.

    Returns:
        An object containing a base64 encoded email object.
    """
    message = MIMEText(message_text)
    message['to'] = to
    message['from'] = "me"
    message['subject'] = subject
    b64_bytes = base64.urlsafe_b64encode(message.as_bytes())
    b64_string = b64_bytes.decode()
    return {'raw': b64_string}


def CreateMessageWithAttachment(to, subject, message_text, file_path):
    """Create a message for an email.

    Args:
        to: Email address of the receiver.
        subject: The subject of the email message.
        message_text: The text of the email message.
        file_path: The path to the file to be attached.

    Returns:
        An object containing a base64 encoded email object.
    """
    message = MIMEMultipart()
    message['to'] = to
    message['from'] = "me"
    message['subject'] = subject

    msg = MIMEText(message_text)
    message.attach(msg)

    content_type, encoding = mimetypes.guess_type(file_path)

    if content_type is None or encoding is not None:
        content_type = 'application/octet-stream'
    main_type, sub_type = content_type.split('/', 1)
    if main_type == 'text':
        fp = open(file_path, 'rb')
        msg = MIMEText(fp.read(), _subtype=sub_type)
        fp.close()
    elif main_type == 'image':
        fp = open(file_path, 'rb')
        msg = MIMEImage(fp.read(), _subtype=sub_type)
        fp.close()
    elif main_type == 'audio':
        fp = open(file_path, 'rb')
        msg = MIMEAudio(fp.read(), _subtype=sub_type)
        fp.close()
    else:
        fp = open(file_path, 'rb')
        msg = MIMEBase(main_type, sub_type)
        msg.set_payload(fp.read())
        fp.close()

    msg.add_header('Content-Disposition', 'attachment',
                   filename=os.path.basename(file_path))
    message.attach(msg)

    b64_bytes = base64.urlsafe_b64encode(message.as_bytes())
    b64_string = b64_bytes.decode()
    return {'raw': b64_string}



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="snoozingmail",
    version="0.0.4",
    author="Nick Is Snoozin",
    author_email="snoozinforabrusin@gmail.com",
    description="A minimal python3 wrapper for the Gmail API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nknaian/snoozingmail",
    packages=setuptools.find_packages(),
    install_requires=[
        'google-api-python-client',
        'google-auth-httplib2',
        'google-auth-oauthlib'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.5',
)